{ 
/* FF.AHK -- Fuzzy Filter
Author:     Slobodan Vujnovic 
Purpose:    FF is a wrapper around ZTree's Filter dialog permitting faster 
            and more complex filters with a minium of keystrokes. 
            The resulting "fuzzyspec" is a perfectly legal ZTree Filespec. 
Docs:       http://vujnovic.free.fr/ztw/ztw_ahk01.html
Forum:      http://www.ztw3.com/forum/forum_entry.php?id=104767
Installation and activation:
            Put FF.EXE in any directory and run it once. To load FF at Windows 
            startup, create a shortcut such as:
                %USERPROFILE%\Start Menu\Programs\Startup\FuzzyFilter.lnk
            with Target pointing to the executable:
                C:\1\FF.EXE
            or, if you are modifying the FF source script often:
                C:\1\AHK\AutoHotkey.exe /f C:\1\FF.ahk
            This allows using the systray icon to edit, reload, suspend, or exit the script.
Side-effects: The Windows clipboard is used by FF, hence previous contents is lost 
1. Compiling if AutoHotkey compiler is installed in C:\1\AHK (it's portable!):
            (Normally it's installed in C:\Program Files\AutoHotkey)
DIR C:\1\FF.exe & DEL C:\1\FF.exe
C:\1\AHK\Compiler\Ahk2Exe.exe /in C:\1\FF.AHK /out C:\1\FF.exe /icon C:\1\FF.ICO
DEL C:\1\FF.exe & C:\1\AHK\Compiler\Ahk2Exe.exe /in C:\1\FF.AHK /out C:\1\FF.exe /icon C:\1\FF.ICO & sleep 3 & FF.EXE 
2. Compiling FF from ZTree F9. (If #ZTTools points to "C:\1"):
#ZTTools\AHK\Compiler\Ahk2Exe.exe /in #ZTTools\FF.AHK /out #ZTTools\FF.EXE  /icon C:\1\FF.ICO
3. Compiling FF from ZTree eXecute:
cmd /c C:\1\AHK\Compiler\Ahk2Exe.exe /in C:\1\FF.AHK /out C:\1\FF.exe /icon C:\1\FF.ICO & cmd /c sleep 2 & start "" C:\1\FF.EXE

History: 
    20070215 SV Creation of EZFF
                http://vujnovic.free.fr/ztw/ztw_ahk.html#Script_2
    20090109 SV Created a simpler subset of EZFF, called FF
                http://vujnovic.free.fr/ztw/ztw_ahk.html#Script_1
    20090113 SV Added logic for "abc." case to select files with no extension
    20090114 SV Any number of Name chunks can be entered: abc -.log -junk ab.
    20090118 SV Added tokens: :x, <N, >N
    20090119 SV Added Help popup and web docs
    20090128 SV Added exiting FF, switching ZTree sessions
    20090417 SV Cleanup
    20091011 SV About no longer shown at startup
    20100329 SV Fixed bug in YEAR filter :A
    20120803 SV Changed name to Fuzzy Filter

BUG: :: tries to remove from history, should be treated as *.* 
BUG: Shift+Enter in DW relogs drive - sends Home,*
BUG:  

TODO: 
TODO: Help popup
TODO:   add [More examples] button
TODO: 
TODO: About popup
TODO:   add switch to start with About
TODO:   add Refresh to About
TODO:   add [More...] button to About ZTree section to show config items
TODO: tokenize_input()
TODO: parse WIN.INI to find date format?
TODO: StrTrimCompress
TODO: find use for f<NULL> input. -help?
TODO: +abc      append to Name part
TODO: Move popup to corner
    A_ScreenWidth
    A_ScreenHeight
TODO: 
TODO: 

---------------------------------------------------------
; AHK (Autohotkey) script for easy FARR win resize, by nod5
^CapsLock::
WinGetPos, , ,w,H,A      
WinGet, active_p, ProcessName, A
if active_p = FindAndRunRobot.exe 
 {
  if (W != 258)
   WinMove, A, , , , 258,
  else
   WinMove, A, , , , 450,
 }
Return
---------------------------------------------------------

*/ 
}

ThisVersion             := "20120803"
ThisProduct             := "FF - Fuzzy Filter"
StringSplit, ThisFile, A_ScriptName, .
; Configure AHK operation
#SingleInstance force   ; Skips dialog box, replaces the old instance automatically
                        ; which is similar in effect to the Reload command.
SetWinDelay, 200        ; Sets delay that will occur after each windowing command
SetTitleMatchMode, slow ; http://www.autohotkey.com/docs/commands/SetTitleMatchMode.htm

; Tell user if no ZTree instance found, but remain in memory
; GroupAdd, GroupName [, WinTitle, WinText, Label, ExcludeTitle, ExcludeText]
SetTitleMatchMode, 2    ; String may be anywhere in title (1=start, 3=exact)
GroupAdd, g_ZTree,      ZTreeWin ahk_class ConsoleWindowClass,,,.INI
;About()                ; Show on startup

; Init global variables
DebugMode   = 0     ; OFF
;DebugMode   = 1     ; ON
FFilespec   :=      ; User input with Name parts and tokens
LastFF      :=      ; Saves last FFilespec
Zfilespec   :=      ; Filespec actually executed by ZTree
NameSpec    := "*.*"
LastName    := ""   ; Save NameSpec
MinSizeSpec :=
MaxSizeSpec :=
DateSpec    :=

; Define this GLOBAL hotkey when heavily developing so you can
; remove FF.EXE from memory without clicking in systray like an idiot
; <#\::ExitApp            ; Win+\ Remove FF.EXE from memory
; Make FF hotkeys active ONLY in ZTree windows
#IfWinActive ahk_group g_ZTree
; Define hotkeys
+Enter::MakeFF()    ; Shift+Enter = Launch Fuzzyspec construction
return
;;;;;;;;;;;; End of autoexecute section (done only once) ;;;;;;;;;;;;

#Include \1\GetZTenv.AHK        ; Include file for any script for ZTree enhancing
#Include \1\FF_About.AHK        ; About and Help popups

DebugModeToggle()
{
global DebugMode
static OnOff := 1
OnOff := !OnOff
DebugMode := OnOff
;DebugPopup()    ; Update DEBUG popup, if visible
}

MakeFF() ; Construct fuzzyspec from 4 elements
{
global NameSpec, MinSizeSpec, MaxSizeSpec, DateSpec, LastFF, FFilespec, LastName
Clipboard=
SendInput {Home}+{End}+{Delete} ; Get user FFilespec to Clipboard
Clipwait, 2             ; Wait for Clipboard to be filled
FFilespec := Clipboard
; --- Start of special commands ---
if (FFilespec = "\") {  ; Unload from memory
    SendInput {Esc}     ; Exit Filter dialog
    ExitApp
    return
    }
if (FFilespec = "|") {  ; Cycle ZTree sessions
    SendInput {Esc}     ; Exit Filter dialog
    GroupActivate, g_ZTree ; Find next group window
    return
    }
if (FFilespec = ".") {  ; Fetch last fuzzyspec to Filespec
    FetchFF()         
    return
    }
if (FFilespec = "*") {  ; Fetch last Zfilespec to Filespec
    FetchZspec()         
    return
    }
if (FFilespec = "??") { ; About popup
    SendInput {Esc}     ; Exit Filter dialog
    About()             ; About popup
    return
    }
if (FFilespec = "?") {  ; Help popup
    SendInput {Esc}     ; Exit Filter dialog
    ProductHelp()       ; Product specific help
    return
    }
; --- End of special commands ---
LastFF  := FFilespec   ; Save FFilespec
len     := StrLen(FFilespec)
; MsgBox, FFilespec:`t'%FFilespec%'`tlen:%len%
; Get user input, parse and sprinkle with *
cal         := 0    ; calendar flag
MINsize     := 0    ; MINsize counter
MAXsize     := 0    ; MAXsize counter
d           := 0    ; dot index
i           := 1    ; index
c           := ""   ; Character to parse
w           := "*"  ; Wildcard character to include
LastName    := NameSpec ; Save last, initially *.*
NameSpec    := ""   ; Build new one
Loop, %len% ; Process every character in the string
{
    ; SubStr(String, StartingPos [, Length]) ; starts at 1
    ; ------------------- Start of tokens -------------------
    c := SubStr(FFilespec, i, 1) ; Get next character
    if (cal > 0) {      ; Parse calendar ':x'
        MakeDateSpec(c) ; Construct DateSpec
        w := ""         ; Wildcard character to include
        cal := 0        ; calendar flag
        i := i + 1      ; Next character
        continue        ; Ignore this character
        }
    if (MINsize > 0) {  ; Parse '<N'
        MakeMinSizeSpec(c) ; Construct MinSizeSpec
        w := ""         ; Wildcard character to include
        MINsize  := 0   ; MINsize counter
        i := i + 1      ; Next character
        continue        ; Ignore this character
        }
    if (MAXsize > 0) {  ; Parse '>N'
        MakeMaxSizeSpec(c) ; Construct MaxSizeSpec
        w := ""         ; Wildcard character to include
        MAXsize := 0    ; MAXsize counter
        i := i + 1      ; Next character
        continue        ; Ignore this character
        }
    if (c = ".") 
        d := i          ; Extension part supplied
    ; ------------------- End of tokens -------------------
    if (c = "*") {      ; Superfluous '*' seen
        Goto NextChar   ; Next character
        }
    if (c = ":") {      ; ':' token
        cal := cal+1
        Goto NextChar   ; Next character
        }
    if (c = ">") {      ; '>' token
        MAXsize := MAXsize+1
        Goto NextChar   ; Next character
        }
    if (c = "<") {      ; '>' token
        MINsize := MINsize+1
        Goto NextChar   ; Next character
        }
    if (c = "/") {      ; Suppress '*' at this position
        w := ""         ; Wildcard character to include
        Goto NextChar   ; Next character
        }
    if (c = " ") {      ; New spec detected! Copy char as is
        if (d = 0)      ; No '.' seen
            NameSpec := NameSpec . "*.*"    ; Add final "*.*"
        else if (d < i)                     ; '.' seen before last char
            NameSpec := NameSpec . "*"      ; Add final *
        ; else when '.' is the very last char, do not add anything!
        d := 0          ; dot index
        w := ""         ; Wildcard character to precede it
        }
    if (c = "-") {      ; Copy char as is
        w := ""         ; Wildcard character to precede it
        }
    NameSpec := NameSpec . w . c  ; Build NameSpec
    w := "*"  ; Wildcard character to include
NextChar:   ; Next character
    i := i + 1          ; Next character
; MsgBox, NameSpec in loop:`t'%NameSpec%'`td:%d%`ti:%i%`tchar:%c%
} ; Loop end
; MsgBox, NameSpec loop exit:`t'%NameSpec%'`td:%d%`ti:%i%

; Add final wildcards
if (NameSpec <> "")         ; Check if NameSpec is empty
    {
    if (d = 0)              ; No '.' seen
        NameSpec := NameSpec . "*.*"    ; Add final "*.*"
    else if (d < i-1)       ; '.' seen before last char
        NameSpec := NameSpec . "*"      ; Add final *
    ; else when '.' is the very last char, do not add anything!
    }
else
    NameSpec := LastName    ; No new NameSpec given, use last
LastName := NameSpec    ; Save current NameSpec
DebugPopup()    ; Debug popup
SendFF()        ; Finally, send fuzzyspec to ZTree to execute the real Filespec!
}

DebugPopup() 
{
global NameSpec, MinSizeSpec, MaxSizeSpec, DateSpec, LastFF, LastName
global DebugMode, Zfilespec
if (DebugMode = 0) { 
   Progress, Hide
   return
   }
MsgBox,
(
======================= DEBUG =======================
Last Fuzzy Filter:`t%LastFF%
`t(Recall with f.[Shift+Enter])
Last ZTree filespec:`t%Zfilespec%
`t(Recall with f*[Shift+Enter])
`nExpanded FF:`t%NameSpec%%MinSizeSpec%%MaxSizeSpec%%DateSpec%
`tMade up from:
NameSpec:`t%NameSpec%
MinSizeSpec:`t%MinSizeSpec%
MaxSizeSpec:`t%MaxSizeSpec%
DateSpec:`t%DateSpec%
)
return
}

SendFF()    ; Send FF to Filespec
{
global NameSpec, MinSizeSpec, MaxSizeSpec, DateSpec, Zfilespec
global DebugMode, Zfilespec
SendInput %NameSpec%%MinSizeSpec%%MaxSizeSpec%%DateSpec%{Enter}
history = %NameSpec%%MinSizeSpec%%MaxSizeSpec%%DateSpec%
; Cleanup Filespec history, whose use can't be bypassed currently
; ZTree does not store *.* in history, so make sure we
; don't wipe out an innocent bystander entry
if (history <> "*.*") {
    Clipboard=
    SendInput f{Up}{Enter}{Home}+{End}+{Delete} ; Get last Zfilespec to Clipboard
    Clipwait, 2             ; Wait for Clipboard to be filled
; We want to save this in case the user wants to use it later
; Also, ZTree's FILE box is too narrow to show us this
    Zfilespec := Clipboard
    if (DebugMode = 1)
        MsgBox, Deleting from ZTree's Filespec history:`n`n%Zfilespec%`n`n`nRecall with:`n`nf*[Shift+Enter]
    SendInput {Up}{Delete}{Esc}{Esc}   ; Delete from history
    }
return
}

MakeDateSpec(c)   ; Construct DateSpec
; Watch out for = and == comparisons!
{
global DateSpec
; ZTree cannot see below TODAY (24h)
    op := " <"      ; :x = BEFORE
if (c == "Y" || c == "W" || c == "M" || c == "A" || c == "C" || c == "F")
    op := " >="     ; :X = SINCE
if (c = ":")        ; :: set calendar to ALL
    DateSpec := ""
else if (c == "T")  ; :T Today only
    DateSpec := " =TODAY"  
else if (c == "t")  ; :t NOT Today
    DateSpec := " <>TODAY"  
else if (c = "Y")   ; :Y/y yesterday
    DateSpec := op . "TODAY-1"
else if (c = "W") { ; :W/w week
    FormatTime, DOW,,WDay
    DOW--                   ; 1-7 = Sunday - Satruday
    If DOW = 0
        DOW := 7            ; Force 1-7 = Monday - Sunday
    offset := (DOW - 1)     ; How far from Monday are we?
    If offset = 0           ; Monday?
        DateSpec := " =TODAY"
    else
        DateSpec := op . "TODAY-" . offset
    }
else if (c = "M")    ; :M/m month
    DateSpec := op . A_Year . "-" . A_Mon . "-01"
else if (c == "A")   ; :A since year started
    DateSpec := " >=" . A_Year . "-01-01,<=" . A_Year . "-12-31"
else if (c == "a")   ; :a other years
    DateSpec := " <" . A_Year . "-01-01,>" . A_Year . "-12-31"
else if (c == "F")   ; :F Dates in the FUTURE"
    DateSpec :=  " >TODAY"
else if (c == "f")   ; :f Dates NOT in the FUTURE"
    DateSpec :=  " <=TODAY"
else if (c = "C")    ; :C/c this century
    DateSpec := op . "2000-01-01"
; Customize here by adding more values...
else ; Error 
    MsgBox, Invalid ':%c%' token.  Request ignored.`nValid options are:`n`n:T/t :Y/y :W/w :M/m :A/a :C/c :F/f`n:: resets filter
}

MakeMinSizeSpec(c)  ; '<' Construct MinSizeSpec
{
global MinSizeSpec
if      (c = "<")   ; << set MIN size to ALL
    MinSizeSpec := ""  
else if (c = "0")   ; 
    MinSizeSpec := " =s0"  
else if (c = "1")   ; 
    MinSizeSpec := " <=s10"  
else if (c = "2")   ; 
    MinSizeSpec := " <=s100"  
else if (c = "3")   ; 
    MinSizeSpec := " <=s1000"  
else if (c = "4")   ; 
    MinSizeSpec := " <=s10000"  
else if (c = "5")   ; 
    MinSizeSpec := " <=s100000"  
else if (c = "6")   ; 
    MinSizeSpec := " <=s1000000"  
else if (c = "7")   ; 
    MinSizeSpec := " <=s10000000"  
else if (c = "8")   ; 
    MinSizeSpec := " <=s100000000"  
else if (c = "9")   ; 
    MinSizeSpec := " <=s1000000000"  
; Customize here by adding more values...
else ; Error 
    MsgBox, Invalid '<%c%' token.  Request ignored.`nValid options are:`n`n<1 <2 ... <9`n\\ resets filter
}

MakeMaxSizeSpec(c)  ; '>' Construct MaxSizeSpec
{
global MaxSizeSpec
if      (c = ">")   ; >> set MAX size to ALL
    MaxSizeSpec := ""  
else if (c = "0")   ; 1000K = 1MB
    MaxSizeSpec := " <>s0"  
else if (c = "1")   ; 
    MaxSizeSpec := " >=s10"  
else if (c = "2")   ; 
    MaxSizeSpec := " >=s100"  
else if (c = "3")   ; 
    MaxSizeSpec := " >=s1000"  
else if (c = "4")   ; 
    MaxSizeSpec := " >=s10000"  
else if (c = "5")   ; 
    MaxSizeSpec := " >=s100000"  
else if (c = "6")   ; 
    MaxSizeSpec := " >=s1000000"  
else if (c = "7")   ; 
    MaxSizeSpec := " >=s10000000"  
else if (c = "8")   ; 
    MaxSizeSpec := " >=s100000000"  
else if (c = "9")   ; 
    MaxSizeSpec := " >=s1000000000"  
; Customize here by adding more values...
else ; Error 
    MsgBox, Invalid '>%c%' token.  Request ignored.`nValid options are:`n`n>1 >2 ... >0`n>> resets filter
}

FetchFF()       ; Paste last fuzzyspec into ZTree Filespec's prompt and wait
{
global LastFF
SendInput {Esc}{Esc}{Esc}f%LastFF% ; Go to FW, insert tokens, wait
return
}

FetchZspec()    ; Paste last Zfilespec into ZTree Filespec's prompt and wait
{
global NameSpec, MinSizeSpec, MaxSizeSpec, DateSpec, Zfilespec
global ZThome, ZTtemp, ZTini ; Must be declared to be seen!
GetZTenv()          ; Get ZTree environment info: ZThome, etc.
/*
    We are squatting here to debug variables and such.
    Don't forget to comment out MsgBox after testing!
    Global variables must be declared to be seen!
#ZTIniPath=C:\ZTW\ZTW_Slo.INI
#ZTTemp=C:\Temp\ZTMP001
*/
;MsgBox, 0, Debugging functions and variables, ZTree variables:`n#ZThome=%ZThome% `n#ZTTemp=%ZTtemp% `n#ZTIniPath=%ZTini%
SendInput {Esc}{Esc}{Esc}f%Zfilespec% ; Go to FW, insert fuzzyspec, wait
return
}
