{
; FF_About.AHK -- Include file for FF.AHK
/* 
http://www.autohotkey.com/docs/commands/Gui.htm#PosPlus
For X and Y, an optional plus sign can be included to position a control 
relative to the right or bottom edge (respectively) of the control that 
was previously added. For example, specifying Y+10 would position the 
control 10 pixels beneath the bottom of the previous control rather 
than using the standard padding distance. Similarly, specifying X+10 
would position the control 10 pixels to the right of the previous 
control's right edge. 

Since negative numbers such as X-10 are reserved for absolute positioning,
to use a negative offset, include a plus sign in front of it. For example: X+-10

xp+n, yp+n, xp-n, yp-n (where n is any number) can be used to position 
controls relative to the previous control's upper left corner, which 
is often useful for enclosing controls in a GroupBox.

xm and ym can be used to position a control at the leftmost and topmost 
margins of the window, respectively (these two may also be followed by 
a plus/minus sign and a number). By specifying ym without any x-position 
at all, the control will be positioned at the top margin but to the right 
of all previously added controls, which can be thought of as starting 
a new "column". The converse is also true.

xs and ys: these are similar to xm and ym except that they refer to 
coordinates that were saved by having previously added a control with 
the word Section in its options (the first control of the window always 
starts a new section, even if that word isn't specified in its options). 
By specifying ys without any x-position at all, the control will be 
positioned at the previously saved y-position, but to the right of all 
controls that have been added since the most recent use of the word 
Section; this can be thought of as starting a new column within the section. 
For example:
gui, add, edit, w600  ; Add a fairly wide edit control at the top of the window.
gui, add, text, section, First Name:  ; Save this control's position and start a new section.
gui, add, text,, Last Name:
gui, add, edit, ys  ; Start a new column within this section.
gui, add, edit
gui, show

The converse of the above (specifying xs but omitting the y-position) is 
also true. xs and ys may optionally be followed by a plus/minus sign and 
a number. Also, it is possible to specify both the word Section and 
xs/ys in a control's options; this uses the previous section for itself 
but establishes a new section for subsequent controls.
*/ 
}

About()
{
; TODO: add DEBUG/SAFE_MODE state and hotkeys
; http://www.autohotkey.com/docs/commands/Gui.htm#Font
global ThisFile1, ThisVersion, ThisProduct, ZTVersion, DebugMode, ZThome, ZTini, ZTtemp
GetZTenv()        ; Get ZTree environment info: #ZThome, etc.
Gui,Destroy
Gui,Add,Picture,Icon1,%ThisFile1%.exe
Gui,Font,Bold s12, Courier New      ; Set font
Gui,Add,Text,x+40 yp+10,%ThisProduct%
Gui,Font,Norm s10                   ; Set font size to 10
Gui,Add,Text,y+0,
;---------------------------------------- FF
Gui,Font,Norm CGreen Bold
Gui,Add,Text,xm,FF.EXE
Gui,Font,Norm
Gui,Add,Text,xp100,Version: 
Gui,Font,w550 CRed
Gui,Add,Text,x+2, %ThisVersion%
Gui,Font,Norm CBlue Underline
Gui,Add,Text,xp100 gURL_FFKIT,Download
;---------------------------------------- Text block
Gui,Font,Normal CBlack
Gui,Add,Text,xm,Welcome to FF, a ZTree Filter enhancer!
Gui,Add,Text,xm,* Click [Help] for FF quick reference
Gui,Add,Text,xm,* Click [Unload] to remove FF from memory
Gui,Add,Text,xm,* Click [Debug] to toggle verbose messages
Gui,Add,Text,xm,From ZTree, you can from now on:
;Gui,Add,Text,xm,* Bring this About popup: f,??,Shift+Enter
Gui,Add,Text,xm,* Show this About popup: f??
Gui,Font,Norm CGreen Bold
Gui,Add,Text,x+0,Shift+Enter
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,xm,* See FF quick reference: f?
Gui,Font,Norm CGreen Bold
Gui,Add,Text,x+0,Shift+Enter
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,xm,* Unload FF from memory: f\
Gui,Font,Norm CGreen Bold
Gui,Add,Text,x+0,Shift+Enter
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,xm,* Switch ZTree instances: f|
Gui,Font,Norm CGreen Bold
Gui,Add,Text,x+0,Shift+Enter
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,xm,* ...and do much more (
Gui,Font,Norm CBlue Underline
Gui,Add,Text,x+0 gURL_FFDOC,full description
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,x+0,)      ; Append to line
;Gui,Add,Text,xm,-------------------------------------------
/* Example: Gui, Add, Checkbox, vShipToBillingAddress, Ship to billing address?

The last parameter is a label displayed next to the box, which is typically used as a prompt or description of what the checkbox does. It may include linefeeds (`n) to start new lines. If a width (W) is specified in Options but no rows (R) or height (H), the control's text will be word-wrapped as needed, and the control's height will be set automatically. The checkbox's associated output variable (if any) receives the number 1 for checked, 0 for unchecked, and -1 for gray/indeterminate.

Specify the word Check3 in Options to enable a third state that displays a gray checkmark instead of a black one (the gray state indicates that the checkbox is neither checked nor unchecked). Specify the word Checked or CheckedGray in Options to have the checkbox start off with a black or gray checkmark, respectively. The word Checked may optionally be followed immediately by a 0, 1, or -1 to indicate the starting state. In other words, "Checked" and "Checked%VarContainingOne%" are the same.

A g-label such as gMySubroutine may be listed in the control's options. This would cause the MySubroutine label to be launched automatically whenever the user clicks or changes the checkbox.

Known limitation: Certain desktop themes might not display a button's text properly. If this occurs, try including -Wrap (minus Wrap) in the button's options. However, this also prevents having more than one line of text.
*/

Gui,Add,Text,xm,___________________________________________
;Gui,Add,Text,y+0,      ; Empty line
;---------------------------------------- ZTree
Gui,Font,Norm Bold CGreen
Gui,Add,Text,xm,ZTW.EXE
Gui,Font,Norm
Gui,Add,Text,xp100,Version: 
Gui,Font,w550 CRed
Gui,Add,Text,x+2, %ZTVersion%
Gui,Add,Text,x+2, (See title)
Gui,Font,Norm CBlue Underline
Gui,Add,Text,xp100 gURL_ZTW4,Download
Gui,Font,Norm CBlack
Gui,Add,Text,xm,#ZThome=%ZThome% 
Gui,Add,Text,xm,#ZTTemp=%ZTtemp%
Gui,Add,Text,xm,#ZTIniPath=%ZTini%
Gui,Font,Norm CBlue Underline
Gui,Add,Text,xm gURL_ZTW,Homepage
Gui,Add,Text,x+10 gURL_ZTW2,&Forum
Gui,Add,Text,x+10 gURL_ZTW3,&Wiki
Gui,Add,Text,x+10 gURL_ZEN,&ZEN
Gui,Font,Norm CBlack,Courier New
;Gui,Add,Text,xm,-------------------------------------------
Gui,Add,Text,xm,___________________________________________
;Gui,Add,Text,y+0,
;---------------------------------------- AHK
Gui,Font,Norm Bold CGreen
Gui,Add,Text,xm,AutoHotkey
Gui,Font,Norm
Gui,Add,Text,xp100,Version: 
Gui,Font,w550 CRed
Gui,Add,Text,x+2, %A_AhkVersion%
;Gui,Font,Norm CBlack,Courier New
Gui,Font,Norm CBlue Underline,Courier New
Gui,Add,Text,xp100 gURL_AHK9,Download
Gui,Font,Norm CBlack Italic
Gui,Add,Text,xm,FF is written in AutoHotkey, learn it:
Gui,Font,Norm CBlue Underline
Gui,Add,Text,xm gURL_AHK,Homepage
Gui,Add,Text,x+10 gURL_AHK3,Wiki
Gui,Add,Text,x+10 gURL_AHK2,Forum
Gui,Add,Text,x+10 gURL_AHK4,Scripts
Gui,Add,Text,x+10 gURL_AHK5,Docs
Gui,Add,Text,x+10 gURL_AHK6,Commands
Gui,Add,Text,xm gURL_AHK7,Variables and Expressions
Gui,Add,Text,x+10 gURL_AHK8,Keys and Clicks
Gui,Font,Norm CBlack,Courier New
Gui,Add,Text,xm,            ; New line
;---------------------------------------- Buttons on bottom
;Gui,Add,Button,% "Default gOK w75 y+10 xp+" (BoxW / 2) - 38,OK 
; Gui,Add,Button,GABOUT_HELP y+0 x+0 w75,&Help
Gui,Add,Button,gABOUT_CLOSE Default w70,&Close
Gui,Add,Button,gABOUT_HELP x+16 w70,&Help
;Gui,Add,Button,gABOUT_DEBUG x+16 w70,&Debug
Gui,Add,Button,gABOUT_UNLOAD x+16 w70,&Unload
Gui,Add,Checkbox,gABOUT_DEBUG x+16,&Debug
;GuiControl,gABOUT_DEBUG,DebugMode
GuiControl,gABOUT_DEBUG,Checked 1
If A_IsCompiled <> 1 ; Do not reload the .EXE version
    Gui,Add,Button,gABOUT_RELOAD x+16 w70,&Reload
Gui,Show,,%ThisProduct% (%ThisVersion%)         About
hCurs:=DllCall("LoadCursor","UInt",NULL,"Int",32649,"UInt") ;IDC_HAND
; OnMessage(0x200,"WM_MOUSEMOVE") 
;IfWinNotExist, ahk_group g_ZTree
;    MsgBox FF has been loaded successfully,`nbut no ZTree instance is currently running.`nWaiting...
Return
}

; Links referenced as 'gURL_XXX' in Gui commands
URL_ZTW:
Run,http://www.ztree.com,,UseErrorLevel
Return
URL_ZTW2:
Run,http://www.ztw3.com/forum/forum.php,,UseErrorLevel
Return
URL_ZTW3:
Run,http://www.ztwiki.com,,UseErrorLevel
Return
URL_ZTW4:
Run,http://www.ztree.com/news.htm,,UseErrorLevel
Return
URL_ZEN:
Run,http://vujnovic.free.fr/ztw/index.html,,UseErrorLevel
Return
URL_FFKIT:
Run,http://vujnovic.free.fr/ztw/ff.zip,,UseErrorLevel
Return
URL_FFDOC:
Run,http://vujnovic.free.fr/ztw/ztw_ahk01.html,,UseErrorLevel
Return
URL_AHK:
Run,http://www.autohotkey.com,,UseErrorLevel
Return
URL_AHK2:
Run,http://www.autohotkey.com/forum,,UseErrorLevel
Return
URL_AHK3:
Run,http://www.autohotkey.com/wiki/index.php?title=Main_Page,,UseErrorLevel
Return
URL_AHK4:
Run,http://www.autohotkey.com/wiki/index.php?title=Script_Listing,,UseErrorLevel
Return
URL_AHK5:
Run,http://www.autohotkey.com/docs,,UseErrorLevel
Return
URL_AHK6:
Run,http://www.autohotkey.com/docs/commands.htm,,UseErrorLevel
Return
URL_AHK7:
Run,http://www.autohotkey.com/docs/Variables.htm#BuiltIn,,UseErrorLevel
Return
URL_AHK8:
Run,http://www.autohotkey.com/docs/commands/Send.htm,,UseErrorLevel
Return
URL_AHK9:
Run,http://www.autohotkey.com/download/,,UseErrorLevel
Return

; Buttons
ABOUT_HELP:
ProductHelp()
Return
ABOUT_CLOSE:
Gui,Destroy
OnMessage(0x200,"") 
DllCall("DestroyCursor","Uint",hCur)
;if (DebugMode = 1)
;    DebugPopup()        ; Update DEBUG popup, if visible
Return

WM_MOUSEMOVE(wParam,lParam)
{
  Global hCurs
  MouseGetPos,,,,ctrl
  If ctrl in Static5,Static13,Static14,Static15,Static19
    DllCall("SetCursor","UInt",hCurs)
  Return
}

; GUI,9: -- Generic GUI Text box with [Close]
PopUp(Title,Message,Font="",FontOptions="",WindowColor="")
{
Gui,9:Destroy
Gui,9:Color,%WindowColor%
Gui,9:Font,%FontOptions%,%Font%
Gui,9:Add,Text,,%Message%
Gui,9:Font
GuiControlGet,Text,9:Pos,Static1
Gui,9:Add,Button,% "Default y+5 w75 gABOUT_CLOSE9 xp+",&Close
;Gui,9:Add,Button,% "w75 gABOUT_UNLOAD xp+" (TextW / 2) - 38 ,&Unload
;If A_IsCompiled <> 1 ; Do not reload the .EXE version
;    Gui,9:Add,Button,% "w75 gABOUT_RELOAD xp+" (TextW / 2) - 38 ,&Reload
Gui,9:-MinimizeBox
Gui,9:-MaximizeBox
; SoundPlay,*-1
Gui,9:Show,,%Title%
Gui,9:+LastFound
; WinSet, Transparent, 150, %Title%
WinWaitClose
Gui,9:Destroy
return
}

; Buttons
ABOUT_DEBUG:
DebugModeToggle()
return
ABOUT_UNLOAD:
ExitApp             ; Unload script
return
ABOUT_RELOAD:
;ScriptReload()      ; Reload script
return
ABOUT_CLOSE9:
Gui,9:Destroy       ; Close this GUI
return

ProductHelp()          ; Product specific help
{
; TODO: add DEBUG/SAFE_MODE state
; TODO: 
; TODO: 
global NameSpec, DateSpec, SizeSpec, ThisProduct,  ThisVersion
; FormatTime, Stamp
; StringRight,WeekNo,A_YWeek,2
; 4+256 = 260, swaps Yes/No button focus default
PopUp(ThisProduct . "   (" . ThisVersion . ")                            Help", ;continuation trick
(
"FF is a wrapper around ZTree's Filter dialog that separates and keeps sticky
the three Filespec filters (Name, Size, Date). It lives alongside Filespec and does
not impact its operation or command history. FF modifies the Windows clipboard. 
`n________________________________ Basic usage ___________________________________
Use 'f' to invoke the Filter dialog as usual, but terminate with [Shift+Enter]
to generate a *f*u*zzy*s*pec* from just a few key characters (and FF tokens) you
entered. Examples of fuzzyspecs expanded into perfectly legal ZTree filters:
  zt.x:y -> *z*t*.*x*,>=TODAY-1 (:y means 'modified BEFORE yesterday')
  zt -beta -.log' -> *z*t*.*,-*b*e*t*a*.*,-*.*l*o*g*
  zt. -> *z*t*. (no '*' is appended if '.' is the last character)
`n_____________ Advanced: Optional special characters and FF tokens ______________
1. Use '/' in positions where you do NOT want a '*' to be inserted by FF
  Example: /z/tw>4 -> zt*w*.*,>=s10000 (ZTree product files above 10^4 bytes)
2. Date 'calendar' tokens. For 'SINCE' filtering, use :X, for 'BEFORE', use :x
:T/t`tToday (=TODAY/<>TODAY)`t`t:Y/y`tYesterday (>=TODAY-1/<TODAY-1)
:W/w`tWeek (>=Monday/<Monday)`t`t:M/m`tMonth (>=yyyymm01/<yyyymm01)
:A/a`tyeAr (>=yyyy0101/<yyyy0101)`t:C/c`tCentury (>=20000101/<20000101)
:F/f`tFuture (>TODAY/<=TODAY)`t`t::`tRemove Date filter
  Example: abc. -.tm /z/t:A -> *a*b*c*.,-*.*t*m*,zt*.*,>=2012-01-01 (SINCE yeAr)
3. To filter by decimal size ranges, insert Size tokens anywhere
Digits after tokens stand for the number of zeros (1=10, 2=100,... 9=10^9):
<1..<9`t<=s10..<=s1,000,000,000`t`t<0`t=s0 (special notation)
>1..>9`t>=s10..>=s1,000,000,000`t`t>0`t<>s0 (special notation)
  Example: .avi .iso >8:C -> *.*a*v*i*,*.*i*s*o*,>=s100000000 >=2000-01-01
`n______________ Miscellaneous commands that manage the FF environment ___________
f.`tRecall last fuzzyspec`t`tf*`tRecall expanded filespec
f`tOnly reset Name part to *.*`tf?`tShow this quick reference
f::`tOnly remove Date filter`t`tf??`tShow the About popup
f<<`tOnly remove MIN Size filter`tf|`tSwitch to next ZTree session
f>>`tOnly remove MAX Size filter`tf\`tUnload FF.EXE from memory",
) "Courier New","cYellow Bold s10","Black")
return
}
